// Array Remove - By John Resig (MIT Licensed)
Array.prototype.remove = function(from, to) {
  var rest = this.slice((to || from) + 1 || this.length);
  this.length = from < 0 ? this.length + from : from;
  return this.push.apply(this, rest);
};

var pgalert = function(msg, title, btnText, cb){
	if (this.isPhonegap())
		navigator.notification.alert(msg, cb, title, btnText);
	else
		alert(msg);
};
var pgconfirm = function(title, msg, buttons, callback){
	if (isPhonegap())
		navigator.notification.confirm(msg, callback, title, buttons);
	else{
		if (callback) callback(confirm(msg) ? 2 : 1);
	}
};


var pageHandler = {
	prevPage: null
};
$(function(){
	$(document).on('pagebeforeshow', '.ui-page', function (e, data) {
		console.log('changing from #%s to #%s', data.prevPage.attr('id'), $.mobile.activePage.attr('id'));
		pageHandler.prevPage = data.prevPage.attr('id');
	});

	$("#sessionDetailBackBtn").on('click', function(e){
		$.mobile.changePage("#" + pageHandler.prevPage);
		e.preventDefault();
		return false;
	});
	$("#personDetailBackBtn").on('click', function(e){
		$.mobile.changePage("#" + pageHandler.prevPage);
		e.preventDefault();
		return false;
	});
});

/*
 * UI MODULE
 *
 * Handles all UI bindings, events, rendering, etc; delegates to APP module for data handling.
 */

var ui = ui || {};
ui.templates = {};
ui.render = function(containerSelector, templateName, data){
	var template = Handlebars.templates[templateName];
	$(containerSelector).html(template(data));
};
ui.renderPrepend = function(containerSelector, templateName, data){
	var template = Handlebars.templates[templateName];
	var c = $(containerSelector);
	c.html(template(data) + c.html());
};
ui.renderAppend = function(containerSelector, templateName, data){
	var template = Handlebars.templates[templateName];
	var c = $(containerSelector);
	c.html(c.html() + template(data));
};

ui.fail_handler = function(err, context){
	console.error('[' + context + ']: ', err);
	if (typeof err === 'object'){
		if (app.hasOwnProperty('device_id')) err.deviceId = app.device_id;
		if (app.hasOwnProperty('personId')) err.personId = app.personId;
	}
	// app.reportError('[CURM] JS error: ' + context, err);
}

ui.updateAnnouncements = function(announcements){
	var now = moment();
	var display = _.filter(announcements, function(el){
		var start = moment(el.postDateTime)
		   ,end = moment(el.expireDateTime);
		if (start.isAfter(now) || end.isBefore(now)){
			return false;
		}
		return true;
	});

	//set a readable timestamp
	_.map(display, function(msg){
		msg.displayTime = moment(msg.postDateTime).format('ddd HH:mm a');
		return msg;
	});

	var currentAnnounce = '';
	if (display.length === 0){
		currentAnnounce = 'No announcements at this time...';
		$("#home #announcement").hide();
	}else{
		currentAnnounce = display[0].headline;
		$("#home #announcement").show();
	}

	//update all of the places that the announcement appears
	$("#home #announcement .preview, #stream .wrapper span").html(currentAnnounce);

	ui.render( '#announcementList', 'announcements', { announcements: display } );
};

ui.showSchedule = function(overrideCache, showDate, meridian, searchString, callback){
	meridian = meridian || '*';

	$("#scheduleContent").empty();
	$.mobile.changePage('#schedule');
	$("#schedulePanel").panel("close");

	if (showDate === 'all' && searchString.length === 0){
		ui.render( '#scheduleContent', 'sessions_global', {} );
		var iscroll = $("#schedule").find(".ui-panel-content-wrap .iscroll-wrapper").data("mobileIscrollview");
		iscroll.refresh();
		iscroll.scrollTo(0,0,0,false);
		return;
	}

	$.mobile.loading( 'show' );
	$("#schedule").data('currentDay', showDate);
	$("#schedule").data('meridian', meridian);

	overrideCache = overrideCache || false;
	app.getScheduleData(
		overrideCache
		,showDate
		,meridian
		//success handler
		,function(sessions){

			if (searchString.length > 0){
				//case-insensitive searching
				searchString = searchString.toLowerCase();
				sessions = _.filter( sessions, function(session){
					return (
						session.headingTime //dividers
						|| session.categories.toLowerCase().indexOf(searchString) > -1
						|| session.desc.toLowerCase().indexOf(searchString) > -1
						|| session.title.toLowerCase().indexOf(searchString) > -1
						|| session.location.toLowerCase().indexOf(searchString) > -1
					);
				});

				//now remove time dividers with no sessions in that time slot
				var newSessions = [];
				_.each(sessions, function(session, ix, arr){
					if (session.headingTime && sessions[ix+1] && sessions[ix+1].headingTime){
						//don't push current headingTime into new array because it would show an
						//empty section
					}else{
						newSessions.push(session);
					}
				});

				//and if the last thing in the list is a time divider, remove that too
				if (newSessions[newSessions.length-1].headingTime){
					newSessions.remove(-1);
				}

				sessions = newSessions;
			}

			//display something akin to "no results" if array is empty
			if (sessions.length === 0){
				ui.render( '#scheduleContent', 'sessions_empty', {} );
			}else{
				ui.render( '#scheduleContent', 'sessions', { sessions: sessions } );
			}

			$.mobile.loading( 'hide' );
			var sched = $("#schedule");
			var iscroll = sched.find(".ui-panel-content-wrap .iscroll-wrapper").data("mobileIscrollview");
			iscroll.refresh();
			iscroll.scrollTo(0,0,0,false);

			//attach each schedule row to its session object
			_.each(sessions, function(session){
				if (!session.title) return;
				sched.find(".schedTitle[data-eventId=" + session.id + "]").jqmData(
					'session'
					, session
				);
			});

			var trackPage = 'Sessions: ' + showDate;
			trackPage += (meridian === '*') ? '' : ' ' + meridian;
			searchString = (searchString.length > 0) ? 'search: ' + searchString : '';
			app.queueTracking(app.info.device_id, trackPage, searchString);

			if (callback) return callback();
		}
		//error handler
		, function(err){
			if ($("#schedule .schedRow").length > 0){
				pgalert("Unable to refresh schedule data. Showing latest known data.", "API Error");
			}else{
				pgalert("Unable to load the latest schedule data. Please try again later.", "API Error");
			}
			ui.fail_handler(err, 'ui.showSchedule > app.getScheduleData');
			$.mobile.loading( 'hide' );
		}
	);
};
ui.showSessionDetail = function( sessionData ){
	$("#sessionContent").empty();
	$.mobile.changePage("#sessionDetail");
	$.mobile.loading('show');
	ui.render( '#sessionContent', 'session', { session: sessionData } );
	$("#sessionContent").jqmData('sessionId', sessionData.id);
	ui.checkBookmark( sessionData.id, "#sessionContent .fa-star, #sessionContent .fa-lock" );
	app.queueTracking(app.info.device_id, 'SESSION: ' + sessionData.title);

	$("#sessionContent #activityRating").change(function(){
		var $this = $(this);
		var lbl = $this.find('option:checked').html();
		$this.closest('.ui-btn').find('.ui-btn-inner .ui-btn-text span').html( lbl );
	});

	//render map
	if (sessionData.latitude === '' && sessionData.longitude === ''){
		$("#sessionContent #sessionMap").hide();
		$.mobile.loading('hide');
	}else{
		$("#sessionContent #sessionMap").show();
		setTimeout(function(){
			var myOptions = {
				center: new google.maps.LatLng(sessionData.latitude, sessionData.longitude)
				,zoom: 14
				,mapTypeId: google.maps.MapTypeId.ROADMAP
				,zoomControl: true
				,streetViewControl: false
			};
			var map = new google.maps.Map($("#sessionContent #sessionMap")[0], myOptions);
			var marker = new google.maps.Marker({
				position: new google.maps.LatLng(sessionData.latitude, sessionData.longitude)
				,map: map
				,title: sessionData.title
				,animation: google.maps.Animation.DROP
			});
			google.maps.event.addListener(marker, 'click', function(e){
				pgconfirm('Open In Google Maps','Would you like to open this location in Google Maps?', ['No','Yes'], function(btn){
					if (btn === 2){
						window.open('https://maps.google.com/maps?hl=en&z=19&q=' + sessionData.latitude + ',' + sessionData.longitude, '_system', null);
					}
				});
			});

			$.mobile.loading('hide');
		}, 500);
	}
};

ui.showPeople = function(event, junk, filter, group){
	filter = filter || '';
	group = group || '';

	if (filter === ''){
		//show search prompt
		ui.render( '#peopleContent', 'people', { people: [{ checkedin: false, displayName: "Search by name or class year (required). Optionally limit to a single school." }] } );
		return;
	}else if (filter.length < 2){
		pgalert('Please be more specific. There are a *lot* of people to search!','Insufficient Search');
		return;
	}

	$.mobile.loading( 'show' );
	app.getPeopleData(
		//event object
		null
		//success handler
		, function(people){
			if (people.length === 0){
				people.push({displayName: 'No people found'});
			}
			ui.render( '#peopleContent', 'people', { people: people } );
			$("#peopleContent").closest(".iscroll-wrapper").data("mobileIscrollview").refresh();
			_.each(people, function(person){
				$("#peopleContent li[data-id=" + person.personid + "]").jqmData('person', person);
			});
			$.mobile.loading( 'hide' );
		}
		//error handler
		, function(err){
			pgalert("Not able to load people. Please try again later.", "API Error");
			ui.fail_handler(err, 'ui.showPeople > app.getPeopleData');
		}
		, filter
		, group
	);
};
ui.showPersonDetail = function( person ){
	ui.render('#personDetail .personDetailContent', 'person', { person: person });
	$.mobile.changePage('#personDetail');
}
ui.displayProfile = function(){
	api.get(
		'/profile'
		, { personId: app.personId }
		, function(data){
			ui.render( '.profileContent', 'profile', { person: data } );
		}
		, function(jqXHR, textStatus, errorThrown){
			ui.fail_handler(jqXHR.responseText + '; ' + errorThrown, 'showProfile');
		}
	);
};


ui.checkBookmark = function(sessionId, bookmarkSelector){
	var setBookmark = function(){
		var found = _.find(app.rawPersonalSchedule, function(item){
			return item.hasOwnProperty('id') && item.id === sessionId;
		}) || false;
		if (found !== false){
			$(bookmarkSelector).removeClass('off');
		}else{
			$(bookmarkSelector).addClass('off');
		}
	};
	if (app.rawPersonalSchedule.length === 0){
		app.getMySchedule(setBookmark, null);
	}else{
		setBookmark();
	}
};

ui.takePhoto = function(){
	app.takePhoto(
		true //use camera
		, function success(file_uri){
			$("#photoUpload #buttons").hide();
			$("#photoUpload .thePhoto").show().find("img").attr('src', file_uri);
		}
		, function fail(err){
			toastr.error('Error getting photo. :(');
		}
	);
};
ui.choosePhoto = function(){
	app.takePhoto(
		false //select from gallery
		, function success(file_uri){
			$("#photoUpload #buttons").hide();
			$("#photoUpload .thePhoto").show().find("img").attr('src', file_uri);
		}
		, function fail(err){
			console.error(err);
			toastr.error('Error getting photo :(');
		}
	);
};
ui.photoUploadSuccess = function(r){
	toastr.success('Photo upload completed');
	// if (ui.gallery.minId !== 9999){
	// 	ui.gallery.prepend();
	// }
	$.mobile.changePage('#gallery');
	$("#photoUpload #buttons").show();
	$("#photoUpload .thePhoto").hide().find('img').attr('src', '');
};
ui.photoUploadFail = function(err){
	toastr.error('Photo upload failed! :(');
	console.error(err);
	ui.fail_handler(err, 'app.uploadPhoto');
};

ui.mySchedule = {};
ui.mySchedule.load = function(){
	$.mobile.loading( 'show' );
	var sched = $("#myScheduleContent");
	app.getMySchedule(
		function(data){
			ui.mySchedule.rawdata = data;
			ui.render( '#myScheduleContent', 'sessions', { sessions: data } );
			ui.mySchedule.iscrollview.refresh();
			$("#myScheduleContent").closest(".iscroll-content").css("padding", 0);
			$.mobile.loading( 'hide' );
			//attach each schedule row to its session object
			_.each(data, function(session){
				if (!session.title) return;
				sched.find(".schedTitle[data-eventId=" + session.id + "]").jqmData(
					'session'
					, session
				);
			});

		}
		, function(err){
			ui.fail_handler(err, 'ui.mySchedule.load');
			if (sched.find(".schedRow").length > 0){
				pgalert('Unable to refresh your personal schedule. Try again later.', 'API Error');
			}else{
				pgalert('Unable to get your personal schedule. Please try again later.', 'API Error');
			}
		}
	);
};

ui.mySchedule.nextUpTimer = null;
ui.mySchedule.nextUpTimerFrequency = 1000 * 60 * 5; //5 minutes
ui.mySchedule.nextUp = function(){
	app.getNextUp(
		function(session){
			$("#home").find(".room .pad").html(session.title).show()
			.end().find(".desc").html(session.location);
		}
		, function(){
			$("#home").find(".desc").html('You don\'t have any more sessions bookmarked. Click the star in the corner of a session detail page to add more.')
			.end().find(".room .pad").html('').hide();
		}
	);
};
ui.mySchedule.startNextUpTimer = function(){
	ui.mySchedule.nextUpTimer = setInterval(ui.mySchedule.nextUp, ui.mySchedule.nextUpTimerFrequency);
};
ui.mySchedule.stopNextUpTimer = function(){
	clearInterval(ui.mySchedule.nextUpTimer);
	ui.mySchedule.nextUpTimer = null;
};

/*
 * PARTY SELECT
 */
ui.party = {};
ui.party.render = function(party){
	ui.render( '#partyList', 'party_select', {people: party} );
};

/**
 * PHOTOS
 */
ui.gallery = {};
ui.gallery.iScrollView = null;
ui.gallery.maxId = 0;
ui.gallery.minId = 9999;
ui.gallery.append = function(event, data){
	app.getPhotos(
		{
			maxRows: 18 //start with 6 rows of 3
			,before_id: ui.gallery.minId
		}
		,function(data){
			if (!data){//304 not modified
				ui.gallery.iScrollView.refresh();
				return;
			}

			if (data.length === 0){
				if (ui.gallery.iScrollView) ui.gallery.iScrollView.refresh();
				return;
			}

			_.each(data, function(el, ix, list){
				if (el.id > ui.gallery.maxId) ui.gallery.maxId = el.id;
				if (el.id < ui.gallery.minId) ui.gallery.minId = el.id;
			});
			ui.renderAppend( '#galleryContent', 'gallery', {photos: data} );
			ui.gallery.iScrollView.refresh();
			ui.gallery.setupSlideshow('gallery');
		}
		,function(err){
			console.error(err);
			toastr.error('There was an error loading more photos...');
		}
	);
};
ui.gallery.prepend = function(event, data){
	app.getPhotos(
		{
			maxRows: 18 //start with 6 rows of 3
			,since_id: ui.gallery.maxId
		}
		,function(data){
			if (!data){//304 not modified
				ui.gallery.iScrollView.refresh();
				return;
			}

			if (data.length === 0){
				if (ui.gallery.iScrollView) ui.gallery.iScrollView.refresh();
				return;
			}

			_.each(data, function(el, ix, list){
				if (el.id > ui.gallery.maxId) ui.gallery.maxId = el.id;
				if (el.id < ui.gallery.minId) ui.gallery.minId = el.id;
			});
			ui.renderPrepend( '#galleryContent', 'gallery', {photos: data} );
			ui.renderPrepend( '#home .slideshow', 'homeGallery', {photos: data} );
			$("#home #photos").show();
			ui.gallery.iScrollView && ui.gallery.iScrollView.refresh();
			ui.gallery.setupSlideshow('gallery');
			ui.gallery.setupSlideshow('home');
		}
		,function(err){
			console.error(err);
			toastr.error('There was an error loading more photos...');
		}
	);
};
ui.gallery.setupSlideshow = function(id){
	var options = {
		enableMouseWheel: false
		,enableKeyboard: false
		,autoStartSlideshow: true
		,captionAndToolbarShowEmptyCaptions: false
		,captionAndToolbarAutoHideDelay: 0 //never hide
		,loop: true
		,nextPreviousSlideSpeed: 500
	};
	var photoSwipeInstance = Code.PhotoSwipe.getInstance(id);
	if (typeof photoSwipeInstance !== 'undefined' && photoSwipeInstance !== null) {
		ui.gallery.teardownSlideshow(id);
	}
	photoSwipeInstance = $(".slideshow a", '#'+id).photoSwipe(options, id);
	console.log('slideshow setup for [%s] complete', id);
};
ui.gallery.teardownSlideshow = function(id){
	var photoSwipeInstance = Code.PhotoSwipe.getInstance(id);
	if (typeof photoSwipeInstance != "undefined" && photoSwipeInstance !== null) {
		Code.PhotoSwipe.detatch(photoSwipeInstance);
	}
};


/*
 * STREAM
 */

ui.stream = {};
ui.stream.maxItem = -1;
ui.stream.minItem = -1;
ui.stream.rate = 30;
ui.stream.updateFrequency = 10 * 1000; //10 seconds
ui.stream.visible = [];
if (localStorage.getItem('likes') === null){
	ui.stream.liked = {};
}else{
	ui.stream.liked = JSON.parse( localStorage.getItem('likes') );
}

ui.stream.init = function(){
	if ($("#streamContent li").length === 0){
		ui.stream.load(function(){
			ui.stream.infScrollView.refresh();
		});
	}else{
		ui.stream.prepend();
	}
};

ui.stream.bindLikeButtons = function(){
	$("#stream .like").not(".bound").addClass("bound").click(function(e){
		e.preventDefault();
		console.log('like button clicked');
		var id = $(this).data('id');
		app.trackClick("Photo Like: " + id);
		$(this).find(".fa").addClass("fa-thumbs-up").removeClass("fa-thumbs-o-up");
		ui.stream.liked[ id ] = true;
		localStorage.setItem( 'likes', JSON.stringify(ui.stream.liked) );
		return false;
	});
};

ui.stream.load = function(callback){
	// if ($("#streamContent li").length > 0) return; //if there's already some content on the page, do nothing
	$.mobile.loading( 'show' );
	app.getStreamData(
		//success handler
		function(data) {
			//restore existing likes
			_.each(data, function(post){
				if (ui.stream.liked.hasOwnProperty(post.id)){
					post.isLiked = true;
				}else{
					post.notLiked = true;
				}
			});

			ui.render( '#streamContent', 'stream', { rows: data } );
			ui.stream.infScrollView.refresh();

			ui.stream.bindLikeButtons();

			if (typeof(data) !== 'undefined' && data.length > 0){
				ui.stream.maxItem = data[0].tsraw;
				ui.stream.minItem = data[data.length-1].tsraw;
			}

			//mark the new posts as visible
			_.each(data, function(post){
				ui.stream.visible.push(post.id);
			});

			$.mobile.loading( 'hide' );

			if (callback) callback();
		}
		//error handler
		, function(err){
			ui.fail_handler(err, 'ui.stream.load > app.getStreamData');
			toastr.error('Error getting stream updates. Please try again later.');
			if (callback) callback();
		}
	);
};

ui.stream.prepend = function(event, data){
	app.getStreamData(
		function(data){
			if (!data){//304 not modified
				ui.stream.infScrollView.refresh();
				return;
			}

			//remove any items that are already visible
			data = _.filter(data, function(item){
				return !_.contains(ui.stream.visible, item.id);
			});

			//clean up when no data to inject
			if (data.length === 0){
				ui.stream.infScrollView.refresh();
				return;
			}

			//mark the new posts as new & visible
			_.each(data, function(post){
				post.isNew = true;
				ui.stream.visible.push(post.id);
			});

			//mark everything already on screen as not-new
			$("#streamContent .new").removeClass("new");

			ui.renderPrepend( '#streamContent', 'stream', { rows: data } );

			ui.stream.bindLikeButtons();

			ui.stream.maxItem = data[0].tsraw;
			ui.stream.infScrollView.refresh();
			// data.iscrollview.scrollToElement(el, time);
		}
		, function(err){
			ui.fail_handler(err, 'ui.stream.prepend > app.getStreamData');
		}
		, ui.stream.maxItem
		, -1
		, ui.stream.rate
	);
};

ui.stream.append = function(event, data){
	app.getStreamData(
		function(data){
			if (!data){//304 not modified
				ui.stream.infScrollView.refresh();
				return;
			}

			//remove any items that are already visible
			data = _.filter(data, function(item){
				return !_.contains(ui.stream.visible, item.id);
			});

			//clean up when no data to inject
			if (data.length === 0){
				ui.stream.infScrollView.refresh();
				return;
			}

			//mark the new posts as new & visible
			_.each(data, function(post){
				post.isNew = true;
				ui.stream.visible.push(post.id);
			});

			//mark everything on screen already as not-new
			$("#streamContent .new").removeClass("new");

			ui.renderAppend( '#streamContent', 'stream', { rows: data } );

			ui.stream.bindLikeButtons();

			ui.stream.minItem = data[data.length-1].tsraw;
			ui.stream.infScrollView.isLoading = false;
			ui.stream.infScrollView.refresh();
		}
		, function(err){
			ui.fail_handler(err, 'ui.stream.append > app.getStreamData');
		}
		, -1
		, ui.stream.minItem
		, ui.stream.rate
	);
};

ui.stream.truncate = function(event, data, survivors){
	survivors = survivors || ui.stream.rate;
	$("#streamContent>li:gt(" + --survivors + ")").remove();
	survivors = $("#streamContent li");
	ui.stream.maxItem = $(survivors[0]).attr('data-tsraw');
	ui.stream.minItem = $(survivors[survivors.length-1]).attr('data-tsraw');

	//update visible list
	ui.stream.visible = [];
	_.each(survivors, function(post){
		ui.stream.visible.push(parseInt($(post).attr('data-id')));
	});
};


/*
 * GeoRewards
 */
ui.listGeoRewards = function(){
    ui.render("#geoRewardsListContent", "rewards_list", {rewards: app.beacons.beacons});
};
ui.displayGeoReward = function(beacon){
	$("#geoReward .pageTitle").html(beacon.ScreenTitle).data('beacon', beacon);
	ui.render("#geoReward #geoRewardContent", "reward", { content: beacon.ScreenContent, single: beacon.isSingleUse, redeemed: beacon.isRedeemed });
	$("#geoReward #btnRedeem").click(function(){
		var beacon = $("#geoReward .pageTitle").data('beacon');
		console.log('(ui) attempting to redeem beacon');
		if (beacon.isSingleUse && !beacon.isRedeemed){
			app.redeemBeacon( beacon.guid, function(success){
				if (success){
					console.log('(ui) beacon redemption successful');
					$("#geoReward #btnRedeem").remove();
					var now = moment().format('MMM DD, hh:mm A');
					ui.render("#geoReward #geoRewardContent", "reward", { content: beacon.ScreenContent, single: 1, redeemed: 1, redemptionTS: now });
					toastr.success('Redeemed!');
					_.each(app.beacons.beacons, function(b){
						if (b.guid == beacon.guid && b.major == beacon.major && b.minor == beacon.minor){
							b.isRedeemed = 1;
							b.redemptionTS = now;
						}
					});
				}else{
					toastr.error('Please try again', 'Error redeeming beacon');
				}
			});
		}
	});

	$.mobile.changePage('#geoReward');
};

$(function(){

	//
	//all pages
	//
	$(document).on('pagechange', function(e, data){
		var location = data.toPage[0].id;
		//black-list of pages not to track by their name
		if ($.inArray(location, ['login','personSelect','static','staticDetail', 'schedule']) > -1) return;
		if (location === 'sessionDetail'){
			return;
		}
		if (location.length === 0) return;
		app.queueTracking(app.info.device_id, location);
	});

	$('.openLinksExternally').on('click', 'a', function(){
		var url = $(this).attr('href');
		if ($(this).attr('rel') === 'plugin'){
			return true;
		}
		window.open(url, '_system', null);
		return false;
	});

	//
	//home
	//
	$("#home #announcement").on('click', function(){
		$.mobile.changePage('#announcements');
	});

	//
	//social connections
	//
	$(".twitterLogin").on('click', function(){
		social.twitter.login();
	});
	$(".linkedinLogin").on('click', function(){
		social.linkedin.login();
	});
	$(".facebookLogin").on('click', function(){
		social.facebook.login();
	});

	//
	//camera activations
	//
	$(".activateCamera").on('click', function(e){e.preventDefault();}).on('vclick', function(e){
		e.preventDefault();
		if (isAndroid()){
			ui.choosePhoto();
		}else{
			ui.takePhoto();
		}	});
	$("#choosePhoto,.activateChooser").on('click', function(e){e.preventDefault();}).on('vclick', function(e){
		e.preventDefault();
		ui.choosePhoto();
	});

	//
	//schedule
	//
	$(document).on("pageinit", "#schedule", function(event){
		$(".iscroll-wrapper", this).bind({
			iscroll_onpulldown: function(event, data){
				var showDate = $("#schedule #scheduleDay option:checked").val();
				var showMeridian = $("#schedule #scheduleDay option:checked").data('meridian');
				var scheduleQ = $("#schedule #scheduleQ").val();
				ui.showSchedule(true, showDate, showMeridian, scheduleQ, function(){
					data.iscrollview.refresh();
				});
			}
		});
	});

	//keyboard-based submit of schedule search
	$("#schedule #scheduleSearchForm").on("submit", function(e){
		e.preventDefault();
		var showDate = $("#schedule #scheduleDay option:checked").val();
		var showMeridian = $("#schedule #scheduleDay option:checked").data('meridian');
		var scheduleQ = $("#schedule #scheduleQ").val();
		ui.showSchedule(false, showDate, showMeridian, scheduleQ, function(){
			$("#scheduleContent").closest('.iscroll-wrapper').data('mobileIscrollview').refresh();
		});
		return false;
	});

	//update the schedule display when the dropdown changes
	$("#schedule #scheduleDay").on("change", function(){
		$("#schedule #scheduleSearchForm").submit();
	});

	$("#schedule").on("click", ".schedTitle", function(event){
		var $this = $(this);
		var sessionData = $this.jqmData('session');
		ui.showSessionDetail( sessionData );
	});

	$("#home").on("click", ".scheduleBox", function(){
		$.mobile.changePage('#mySchedule');
	});

	//
	//personalized schedule
	//
	$("#mySchedule").on("pageshow", ui.mySchedule.load);

	$(document).on("pageinit", "#mySchedule", function(){
		ui.mySchedule.iscrollview = $(this).find(".ui-panel-content-wrap .iscroll-wrapper").data("mobileIscrollview");

		$(this).find(".iscroll-wrapper").bind({
			iscroll_onpulldown: ui.mySchedule.load
		});
	});

	$("#mySchedule").on("click", ".schedTitle", function(event){
		var $this = $(this);
		var sessionData = $this.jqmData('session');
		ui.showSessionDetail( sessionData );
	});

	//
	//session detail
	//
	$("#sessionDetail").on("vclick", ".fa-star", function(e){
		var $el = $(this);
		var sessionId = $("#sessionContent").jqmData('sessionId');
		$.mobile.loading( 'show' );
		if (!$el.hasClass("off")){
			app.removeSessionFromPersonalSchedule(
				sessionId
				, function (){
					toastr.success("This event has been removed from My Schedule.");
					$el.addClass("off");
					$.mobile.loading( 'hide' );
					ui.mySchedule.nextUp();
				}
				, function(){
					toastr.error("Unable to remove session from My Schedule. Please try again later.", "Bookmark Error");
					$.mobile.loading( 'hide' );
				}
			);
		} else {
			app.addSessionToPersonalSchedule(
				sessionId
				, function (){
					toastr.success("This event has been added to My Schedule.");
					$el.removeClass("off");
					$.mobile.loading( 'hide' );
					ui.mySchedule.nextUp();
				}
				, function (){
					toastr.error("Unable to add session to My Schedule. Please try again later.", "Bookmark Error");
					$.mobile.loading( 'hide' );
				}
			);
		}
	});

	//toasts for registration-required events
	$("#sessionDetail").on("vclick", ".fa-lock", function(){
		var registered = !$("#sessionDetail .fa-lock").hasClass('off');
		var msg = 'Registration is required for this event. Please visit class headquarters to add it to your schedule.';
		if (registered){
			msg = 'Please visit class headquarters to have this event removed from your schedule.';
		}
		toastr.info(msg, 'Registration Required');
	});

	$("#sessionDetail").on("click", "#submitActivityRating", function(e){
		e.preventDefault();
		var sessionId = $("#sessionContent").jqmData('sessionId');
		var rating = $("#activityRating option:selected").val();
		app.queueRating(app.personId, sessionId, rating);
		toastr.success('Evaluation Saved', 'Thanks!');
	});

	//
	//stream
	//
	$(document).on("pageinit", "#stream", function(event){

		//enable infinite scrolling
		ui.stream.infScrollView = $(this).find(".ui-panel-content-wrap .iscroll-wrapper").data('mobileIscrollview');
		ui.stream.tweetH = 75;
		ui.stream.infScrollTimer = setInterval(function(){
			//short circuit if flags aren't twiddled how we'd like
			if (!ui.stream.infScrollView.didScroll || ui.stream.infScrollView.isLoading) return;
			var bottom = ui.stream.infScrollView.iscroll.maxScrollY;
			var trigger = bottom + ui.stream.tweetH*8;
			//short circuit if MATH
			if (ui.stream.infScrollView.iscroll.y > trigger) return;
			ui.stream.infScrollView.didScroll = false;
			ui.stream.infScrollView.isLoading = true;
			ui.stream.infScrollView.$wrapper.trigger(
				'iscroll_onpullup'
				, { iscrollview: ui.stream.infScrollView }
			);
		}
		, 250);

		$(this).find(".iscroll-wrapper").bind({
			iscroll_onpulldown: ui.stream.prepend
			,iscroll_onpullup: ui.stream.append
			,iscroll_onscrollmove: function(event, data){
				data.iscrollview.didScroll = true;
			}
		});

		$("#stream .sticky").on("click", function(){
			$.mobile.changePage("#announcements");
		});
	});

	$(document).on("pageshow", "#stream", function(){
		ui.stream.init();

		//on the regular interval, check for updates
		ui.stream.updateTimer = setInterval(
			ui.stream.prepend
			, ui.stream.updateFrequency
		);
	}).on("pagehide", "#stream", function(){
		ui.stream.truncate();

		//when the page is hidden, stop
		clearInterval(ui.stream.updateTimer);
		ui.stream.updateTimer = null;
	}).on("click", ".share", function(){
		var url = $(this).data('url');
		window.plugins.socialsharing.share(
			social.twitter.hashtag, null, null, url
			,function(success){
				if (success){
					app.trackClick('share: ' + url);
				}else{
					app.trackClick('share cancelled: ' + url);
				}
			}
			,function(err){
				app.trackClick('share error: ' + url);
				console.log(err);
			}
		);
	});

	$("#newMsg").on("pageshow", function(){
		$("#tweetNote").html("We'll automatically append <br/><strong>" + social.twitter.hashtag + "</strong> if you forget it!");
		if (social.twitter.user === null){
			//hide twitter stuff
			$('#msgTo').closest('div').hide();
			$('#charCount').hide();
			$('#tweetNote').hide();
		}else{
			//show twitter stuff
			$('#charCount').show();
			$('#tweetNote').show();
			$('#msgTo').trigger("change").closest('div').show();
		}
	});
	$("#newMsg #msgTo").on("change", function(){
		if ($(this).find('option:checked').val() === 'twitter'){
			$('#charCount').show();
			$('#tweetNote').show();
		}else{
			$('#charCount').hide();
			$('#tweetNote').hide();
		}
	});
	//characters-available counter
	$("#newMsgBody").on('keyup', function(){
		var len = $(this).val().length;
		var color = 'black';
		if (len > 140){
			color = 'red';
		}
		$("#charCount").css('color', color).html(len + '/140');
	});
	$("#newMsgSubmit").on("click", function(e){
		e.preventDefault();
		var msg = $("#newMsgBody").val();
		if (msg.length > 0){

			var dest = $("#msgTo option:checked").val();
			$.mobile.loading( 'show' );
			if (dest === 'twitter'){
				return social.twitter.tweet(msg, function(reply){
					if (reply.httpstatus && reply.httpstatus === 200){
						$.mobile.loading('hide');
						$("#newMsgBody").val('');//clear the textarea
						$.mobile.changePage('#stream');
						toastr.success('Tweet sent');
						return;
					}else{
						$.mobile.loading('hide');
						toastr.error('It looks like there was an error posting your Tweet.');
						setTimeout(function(){ toastr.error('Please check your tweet history and consider trying again.'); }, 1500);
						return;
					}
				});
			}else{
				//post as in-app message
				app.submitMsg(msg);
				$.mobile.loading('hide');
				$("#newMsgBody").val('');
				$.mobile.changePage('#stream');
				toastr.success('Your message will be posted shortly');
				return;
			}
		}
	});


	//
	//people
	//
	$("#people").on("pageshow", ui.showPeople);

	$(document).on("pageinit", "#people", function(event){
		$("#peopleContent").closest(".iscroll-wrapper").bind({
			iscroll_onpulldown: function(event, data){
				var filter = $("#personQ").val();
				var grp = $("#personGrp option:selected").val();
				ui.showPeople(null, null, filter, grp);
				data.iscrollview.refresh();
			}
		});
	});
	$("#people").on("click", ".person", function(){
		var person = $(this).jqmData('person');
		if (!(person == undefined)){
			ui.showPersonDetail( person );
		}
	});
	$("#peopleForm").on("submit", function(e){
		e.preventDefault();
		var filter = $("#personQ").val();
		var grp = $("#personGrp option:selected").val();
		app.queueTracking(app.info.device_id, 'people search: [' + grp + '] ' + filter);
		ui.showPeople(null, null, filter, grp);
		return false;
	});
	$(document).on('click', '.li_connect', function(e){
		e.preventDefault();
		var $panel = $("#personDetail .panel");
		$panel.find("input").val('Join my network on LinkedIn');
		$panel.find("textarea").val("I found you in the Cornell Reunion mobile app");
		$panel.panel('open');
	})
	.on('click', '.liconnect #liconnect_submit', function(){
		var page = $(this).closest('div[data-role=page]');
		var $person = $('.name-badge', page);
		if (social.linkedin.user === null){
			pgconfirm(
				'Login Required'
				,'In order to connect, you must first login to your LinkedIn Account'
				,['Login', 'Never mind']
				,function(buttonIx){
					if (buttonIx === 2){
						return;
					}
					social.linkedin.linkedin.login( social.linkedin.permissions, function(liuser){
						social.linkedin.onLogin( liuser );
						$("#liconnect_submit").click();
					});
				}
			);
			return;
		}
		$.mobile.loading('show');
		social.linkedin.linkedin.connectViaEmail(
			$person.data('fname')
			, $person.data('lname')
			, $person.data('email')
			, $(".liconnect input", page).val()
			, $(".liconnect textarea", page).val()
			, function(err,data){
				$.mobile.loading('hide');
				if (err){
					if (err.invalidToken){
						pgconfirm(
							'LinkedIn Login Expired'
							,'You\'ll need to login to LinkedIn again to continue. LinkedIn only allows connections to last a little while, and it\'s been too long.'
							,['Continue', 'Never mind']
							,function(buttonIx){
								if (buttonIx === 2){
									return;
								}
								//reconnect LinkedIn then resubmit request
								social.linkedin.linkedin.login( social.linkedin.permissions, function(liuser){
									if (user === null){
										return toastr.error('Error connecting to LinkedIn');
									}
									toastr.success('LinkedIn Connected');
									social.linkedin.user = user;
									social.linkedin.displayUser( user );
									app.saveSocial();
									$("body").addClass('li_connected').removeClass('li_disconnected');
									social.mine({
										li_userid:               social.linkedin.user.id
										,li_email:               encodeURIComponent(social.linkedin.user.emailAddress)
										,li_profileUrl:          encodeURIComponent(social.linkedin.user.publicProfileUrl)
										,li_firstname:           encodeURIComponent(social.linkedin.user.firstName)
										,li_lastname:            encodeURIComponent(social.linkedin.user.lastName)
										,li_oauth_access_token:  encodeURIComponent(social.linkedin.linkedin.getToken())
										,li_blob:                encodeURIComponent(JSON.stringify(social.linkedin.user))
									});

									toastr.info('Sending connection request...');
									$("#liconnect_submit").click();
								});
							}
						);
					}else{
						return toastr.error('Error sending connection request. Please try again later.');
					}
				}
				return toastr.success('Connection request sent');
			}
		);
	});


	//
	//photos
	//
	$("#gallery").on("pageshow", function(){
		var wrapper = $(this).find(".ui-panel-content-wrap .iscroll-wrapper");
		ui.gallery.prepend();

		ui.gallery.iScrollView = wrapper.data('mobileIscrollview');

		wrapper.bind({
			iscroll_onpulldown: ui.gallery.prepend
			,iscroll_onpullup: ui.gallery.append
		});
	});
	$("#gallery .slideshow").on('click', 'li', function(){
		var instance = Code.PhotoSwipe.getInstance('gallery');
		var ix = $('#gallery .slideshow li').index($(this));
		instance.show(ix);
	});
	//tech debt: duplicating calls from above and changing id's just to get it done quickly
	$("#home").on("pageshow pageinit", function(){
		ui.gallery.prepend();
	});
	$("#home .slideshow").on('click', 'li', function(){
		var instance = Code.PhotoSwipe.getInstance('home');
		var ix = $('#home .slideshow li').index($(this));
		instance.show(ix);
	});
	$("#gallery").on('click', '.goToPhotoUpload', function(){
		console.log('click');
		$.mobile.changePage('#photoUpload');
	});
	$("#photoUpload").on('click', "#retake", function(){
		$("#photoUpload #buttons").show();
		$("#photoUpload .thePhoto").hide().find('img').attr('src', '');
	})
	.on('click', ".submitPhoto", function(){
		var file_uri = $("#photoUpload .thePhoto img").attr('src')
			,caption = $("#photoUpload #photoCaption").val()
			,lat = $("#photoUpload .lat").val()
			,lng = $("#photoUpload .lng").val()
		app.uploadPhoto(app.userEmail, file_uri, caption, lat, lng, ui.photoUploadSuccess, ui.photoUploadFail);
	})
	.on('click', ".photoGetGeo, .geoLabel", function(){
		var screen = $("#photoUpload");
		var isCurrentlySet = ( screen.find(".geoLabel").html() === 'Remove location' );
		if (isCurrentlySet){
			screen.find('.geoLabel').html('Include location?').closest('.geo').removeClass('captured');
			screen.find('.lat').val('');
			screen.find('.lng').val('');
		}else{
			$.mobile.loading('show');
			app.getGeo(
				/* success */
				function(position){
					$.mobile.loading('hide');
					screen.find('.geoLabel').html('Remove location').closest('.geo').addClass('captured');
					screen.find('.lat').val(position.coords.latitude);
					screen.find('.lng').val(position.coords.longitude);
				}
				/* error */
				,function(error){
					$.mobile.loading('hide');
					toastr.error('Error getting your current location');
				}
			)
		}
	})
	.on('click', ".cancelPhoto", function(){
		$("#photoUpload #buttons").show();
		$("#photoUpload .thePhoto").hide().find('img').attr('src', '');
		$.mobile.changePage('#gallery');
	});

	//
	//static pages
	//
	$("#static").on('pageshow', function(){
		app.queueTracking(app.info.device_id, 'General Info Pages List');
		$.mobile.loading('show');
		app.getPagesList(function(pages){
			ui.render('#staticMenuContent', 'static_list', { pages: pages });
			$.mobile.loading('hide');
		});
	});
	$("#static").on("click", ".staticContentLoader", function(event){
		var key = $(this).attr('data-key');
		app.loadStaticPage(key, function(page){
			$.mobile.changePage('#staticDetail');
			$("#staticDetail .pageTitle").html(page.title);
			$("#staticDetailContent").html(page.content);
			app.queueTracking(app.info.device_id, 'General Info Page: ' + page.title);
		});
	});

	//
	//checkin page qr codes
	//
	$("#checkin").on('pageshow', function(){
		$.mobile.loading('show');
		app.getPartyQRCodes(function(data){
			ui.render('#qrWrapper', 'qr', { people: data });
			$.mobile.loading('hide');
		});
	});


	//
	//geo rewards
	//
	// $("#geoReward").on('click', '#btnRedeem', function(){
	// 	var beacon = $("#geoReward .pageTitle").data('beacon');
	// 	console.log('(ui) attempting to redeem beacon');
	// 	if (beacon.isSingleUse && !beacon.isRedeemed){
	// 		app.redeemBeacon( beacon.guid, function(success){
	// 			if (success){
	// 				console.log('(ui) beacon redemption successful');
	// 				$("#geoReward #btnRedeem").remove();
	// 				ui.render("#geoReward #geoRewardContent", "reward", { content: beacon.ScreenContent, single: 1, redeemed: 1 });
	// 				toastr.success('Redeemed!');
	// 				_.each(app.beacons.beacons, function(b){
	// 					if (b.guid = beacon.guid){
	// 						b.isRedeemed = 1;
	// 					}
	// 				});
	// 			}else{
	// 				toastr.error('Please try again', 'Error redeeming beacon');
	// 			}
	// 		});
	// 	}
	// });
    $("#geoRewardsList").on("pageshow", function(){
        ui.listGeoRewards();
    });
    $("#geoRewardsList").on("click", ".rewardLoader", function(e){
        e.preventDefault();
        var guid = $(this).closest('ul').data('guid');
        var beacon = _.filter(app.beacons.beacons, function(b){
            return b.guid === guid;
        });
        beacon = beacon[0];
        ui.displayGeoReward(beacon);
        return false;
    });

	//
	//reset app
	//
	$("#nuclear").on("click", function(e){
		e.preventDefault();
		app.resetApp();
		pgalert('Your data has been reset. Please log in again.','Reset Complete');
	});

	//
	//feedback form
	//
	$("#sendFeedback").on("vclick click", function(event){
		if (event.type === 'click') return;

		$.mobile.loading('show');
		app.sendFeedback(
			$("#feedbackComments").val()
			,function(){
				$.mobile.loading('hide');
				toastr.success('Feedback sent, thanks!');
				$("#feedbackComments").val('');
			}
			,function(jqXHR, textStatus, errorThrown){
				$.mobile.loading('hide');
				toastr.error('Unable to send feedback. Please try again later.', 'Error');
			}
		);
	});

	//
	//login
	//
	$("#loginform").submit(function(e){
		e.preventDefault();
		$("#submitLogin").click();
		return false;
	});
	$("#submitLogin").on("click", function(e){
		e.preventDefault();
		var email = $("#loginEmail").val()
			,zip = $("#loginZip").val();

		console.log('submitting auth for %s', email);

		$.mobile.loading('show');
		$("#loginMsg").html('').hide();
		app.getParty(
			email
			, zip
			//success
			, function(party){

				$.mobile.loading('hide');
				//if only one person, then skip the person-selection step
				if (party.length === 1){
					console.log('only 1 person in party, skipping person selection...');
					app.registerDevice(
						party[0].personId
						,function(){
							$.mobile.changePage("#connections");
							app.enableStuff();
						}
						,function(msg){
							app.resetApp();
							$("#loginMsg").html(msg).show();
						}
					);
				}else{
					ui.party.render(party);
					$.mobile.changePage('#personSelect');
				}
			}
			, function(msg){
				$.mobile.loading('hide');
				$("#loginMsg").html(msg).show();
			}
		);
	});
	$("#submitPersonSelect").on("click", function(){
		var personId = $("#partyList option:selected").val();

		if (personId === ''){
			pgalert('You must select a person to continue', 'Whoops!');
			return;
		}

		app.registerDevice(
			personId
			//success
			, function(data){
				app.enableStuff();
				//go to social connections
				$.mobile.changePage("#connections");
			}
			//error
			, function(msg){
				app.resetApp();
				$("#loginMsg").html(msg).show();
			}
		);
	});

	//
	//personal profile
	//
	$("#profile").on('pageshow', function(){
		ui.displayProfile();
	});
	$("#profile").on('click', '.profileOptIn', function(){
		$.mobile.loading('show');
		api.post(
			'/optin', { personId: app.personId }
			, function(data){
				$.mobile.loading('hide');
				toastr.success('Your name will appear in the attendee list');
				ui.displayProfile();
			}
			, function(jqXHR, textStatus, errorThrown){
				$.mobile.loading('hide');
				toastr.error('There was an error trying to opt-in. Please try again later.');
			}
		);
	});
	$("#profile").on('click', '.profileOptOut', function(){
		$.mobile.loading('show');
		api.post(
			'/optout', { personId: app.personId }
			, function(data){
				$.mobile.loading('hide');
				toastr.success('Your name no longer appears in the attendee list');
				ui.displayProfile();
			}
			, function(jqXHR, textStatus, errorThrown){
				$.mobile.loading('hide');
				toastr.error('There was an error trying to opt-out. Please try again later.');
			}
		);
	});

	//
	//social connections page
	//
	$("#connections .why").on('click', function(){
		$(this).hide().closest('.well').find('.explained').show();
	});

});
