;var CMSEstimote = function(enableDebug){

	//verify dependencies are met: BackgroundJS plugin
	if (window.hasOwnProperty('plugins') && window.plugins.hasOwnProperty('BackgroundJS')){
		//plugin is here
	}else{
		throw "BackgroundJS plugin not found!";
	}
	//verify dependencies are met: Local Notifications plugin
	if (window.hasOwnProperty('plugin') && window.plugin.hasOwnProperty('notification') && window.plugin.notification.hasOwnProperty('local')){
		//plugin is here
		window.plugin.notification.local.on("click", function(notification, state){
			var data = JSON.parse( notification.data )
			console.log('notification clicked:', notification);
			if (that.notificationHandlers.hasOwnProperty(data.id)){
				that.notificationsVisible[data.id] = false;
				that.notificationHandlers[data.id](data.id, state, data);
			}
		});
	}else{
		throw "Local Notifications plugin not found!";
	}

	this.debug = enableDebug || false;
	this.proximityHandlers = {};
	this.notificationHandlers = {};
	this.notificationsVisible = {};
	this.knownBeacons = {};
	this.notificationCount = 0;

	var that = this;

	return this;
};

CMSEstimote.prototype.off = function(){
	if (this.debug){ console.log('stopping ranging'); }
	window.estimote.nearables.stopRanging();

	this.proximityHandlers = {};
	this.notificationHandlers = {};
	this.knownBeacons = {};
};

//--------------------------------------------------------------

CMSEstimote.prototype.notifyNear = function(beacon, activationCallback, proximityCallback){
	var that = this;

	if (this.knownBeacons.hasOwnProperty( beacon.id )){
		return true;
	}

	var id = beacon.id;
	this.proximityHandlers[ id ] = proximityCallback; //what to do when we're near the beacon
	this.notificationHandlers[ id ] = activationCallback; //what to do when the beacon notification has been clicked
	this.notificationsVisible[ id ] = false;
	this.knownBeacons[ id ] = beacon;

	window.estimote.nearables.startRangingForIdentifier(
		beacon.id

		//when found
		,function(nearable){
			if (that.debug){
				console.log('Nearable found: ');
				window.estimote.printObject( nearable );
			}

			var id = nearable.identifier;
			that.proximityHandlers[ id ]( id );

			//try not to duplicate notifications
			if (that.notificationsVisible[ id ] === true){
				if (that.debug){
					console.log('notification already visible for %s; aborting', id);
				}
				return;
			}

			//SEND UP THE SIGNAL FLARE!!
			if (that.debug){
				console.log('creating notification for %s', id);
			}

			that.notificationCount++;
			console.log('creating notification #' + that.notificationCount);

			that.notificationsVisible[ id ] = true;
			window.plugin.notification.local.schedule({
				id: that.notificationCount
				,title: that.knownBeacons[ id ].notificationTitle
				,text: that.knownBeacons[ id ].notificationBody
				,data: JSON.stringify({ id: id })
			});
		}

		//ranging error
		, function(errMsg){
			if (that.debug){
				console.log('ranging error: ', + errMsg);
			}
		}
	);

	if (that.debug){
		console.log('listener registered for %s', beacon.id);
	}

};
