/*
 * Custom wrapper for LinkedIn specifically for use in Phonegap ( & phonegap build )
 */

;var Linkedin = function(oauth_callback, consumerKey, consumerSecret){

	var loginCallback = null;
	var windowRef = null;
	var user = null;
	var _persister = null;
	var _loader = null;

	var user = {
		access_token: null
		,profile: null
	};

	var credentials = {
		consumerKey: consumerKey
		,consumerSecret: consumerSecret
	};

	//only the first option (location) is used for Android, the rest are for the benefit of iOS
	var loginPopupOptions = [
		'location=yes'
		,'closebuttoncaption=Cancel'
		,'toolbar=yes'
		,'presentationstyle=pagesheet'
		,'transitionstyle=fliphorizontal'
	].join(',');

	function params(the_url){
		var parameters = {}, parameter;
		var query = the_url.split('?');
		if (query.length > 1){
			query = query[1].split('&');
			for (var i = 0; i < query.length; i++) {
				parameter = query[i].split("=");
				if (parameter.length === 1) {
					parameter.push("");
				}
				parameters[decodeURIComponent(parameter[0])] = decodeURIComponent(parameter[1]);
			}
		}
		return parameters;
	}

	function saveUser(newUser){
		user = newUser;
		//if a persistence method is set, call it
		if(_persister !== null) _persister.call(null, user);
	}
	function getUser(){
		return user;
	}

	function onLoadEvent(event){
		var url = event.url;
		var parameters = params(url);

		  console.log('LINKEDIN URL EVENT: %s', url, parameters);

		if (typeof(parameters.code) !== 'undefined'){
			var authorization_code = parameters.code;
			windowRef.close();

			//OAUTH: exchange authorization_code for access token
			var url = "https://www.linkedin.com/uas/oauth2/accessToken"
			        + "?grant_type=authorization_code"
			        + "&code=" + authorization_code
			        + "&redirect_uri=" + encodeURIComponent(oauth_callback)
			        + "&client_id=" + consumerKey
			        + "&client_secret=" + consumerSecret;
			console.log('LINKEDIN: OAUTH: GET ACCESS TOKEN: ' + url);
			$.post(
				url
			).done(function(data, status, xhr){
				if (typeof data.expires_in === 'undefined' || typeof data.access_token === 'undefined'){
					return loginCallback.call(null, null);
				}
				//get profile info
				var access_token = data.access_token;
				saveUser({ access_token: access_token, profile: getUser().profile });

				console.log('LINKEDIN: OAUTH: SUCCESS! WE HAEV TEH CODEZZZ!!!');

				var profileUrl = "https://api.linkedin.com/v1/people/~:(id,first-name,last-name,email-address,headline,public-profile-url,specialties,positions)?format=json&oauth2_access_token=" + access_token;
				// console.log('LINKEDIN: OAUTH: GET PROFILE: ' + profileUrl);
				$.get(
					profileUrl
				).done(function(data, status, xhr){
					console.log('linkedin profile:', data);
					saveUser({ access_token: access_token, profile: data });
					return loginCallback.call(null, data);
				}).fail(function(xhr, status, error){
					console.error('LINKEDIN: profile error [%s][%s]', status, error, xhr);
					return loginCallback(null);
				});
			}).fail(function(xhr, status, error){
				console.error('LINKEDIN: OAUTH: get access token error: ', xhr);
				return loginCallback(null);
			});
		}else if(typeof(parameters.error) !== 'undefined'){
			//login refused
			console.log('LINKEDIN: OAUTH: LOGIN REFUSED', parameters);
			windowRef.close();
			loginCallback.call(null, null);
		}else{
			//probably an interstitial step during the login process; ignore
		}
	}

	var public_interface = {
		login: function(scope, callback){
			scope = scope || '';
			loginCallback = callback;

			//OAUTH: Ask user to grant access
			var url = "https://www.linkedin.com/uas/oauth2/authorization"
			        + "?response_type=code"
			        + "&client_id=" + consumerKey
			        + "&scope=" + encodeURIComponent(scope)
			        + "&state=" + hex_sha1(new Date().toString())
			        + "&redirect_uri=" + encodeURIComponent(oauth_callback);
			console.log('LINKEDIN: OAUTH: PROMPT USER: ' + url);
			windowRef = window.open(url, '_blank', loginPopupOptions);
			windowRef.addEventListener('loadstart', onLoadEvent);
			windowRef.addEventListener('loadstop', onLoadEvent);
		}
		,loadConfigVia: function(fn){
			_loader = fn;
			var userFromDisk = _loader.call(null);
			if (userFromDisk){
				user = userFromDisk;
			}
			return public_interface;
		}
		,persistConfigVia: function(fn){
			_persister = fn;
			if (user){
				_persister.call(null, user);
			}
			return public_interface;
		}
		,getProfile: function(){
			return user.profile;
		}
		,getToken: function(){
			return user.access_token;
		}
		,connectViaEmail: function(fname, lname, email, title, body, callback){
			var url = 'https://api.linkedin.com/v1/people/~/mailbox?format=json&oauth2_access_token=' + user.access_token;
			var data = {
				"recipients": {
					"values": [
						{
							"person": {
								"_path": "/people/email=" + email,
								"first-name": fname,
								"last-name": lname
							}
						}
					]
				},
				"subject": title,
				"body": body,
				"item-content": {
					"invitation-request": {
						"connect-type": "friend"
					}
				}
			};
			$.ajax({
				url: url
				,data: JSON.stringify(data)
				,type: 'POST'
				,contentType: 'application/json'
			}).done(function(data){
				if (callback) callback(null, data);
			}).fail(function(xhr, status, error){
				var err = {xhr:xhr,status:status,error:error};
				if (xhr.status === 401){
					//token expired/revoked/etc
					err.invalidToken = true;
				}
				if (callback) callback(err);
			});
		}
		,joinGroup: function(groupId, callback){
			var url = 'https://api.linkedin.com/v1/people/~/group-memberships/' + groupId + '?format=json&oauth2_access_token=' + user.access_token;
			var body = '<group-membership><membership-state><code>member</code></membership-state></group-membership>';
			$.ajax({
				url: url
				,data: body
				,type: 'PUT'
			}).done(function(data){
				if (callback) callback(null, data);
			}).fail(function(xhr,status,error){
				var err = {error:error,status:status};
				if (xhr.status === 401){
					//token expired/revoked/etc
					err.invalidToken = true;
				}
				if (callback) callback(err);
			});
		}
		// ,mockError: function(){
		// 	$.ajax({
		// 		url: 'http://gw.local/api/v1/index.cfm/err/401'
		// 		,data: null
		// 		,type: 'POST'
		// 	}).done(function(data){
		// 		console.log('success?!');
		// 	}).fail(function(xhr,status,error){
		// 		console.error('status: %s\nerror: %s\n', status, error, xhr);
		// 	});
		// }
	};

	return public_interface;
};
