/*
 * Custom wrapper for codebird specifically for use in Phonegap ( & phonegap build )
 */

;var Twitter = function(oauth_callback, consumerKey, consumerSecret){
	var cb = new Codebird;
	cb.setProxy('https://twitter-cors-proxy.herokuapp.com/index.php');
	cb.setConsumerKey(consumerKey, consumerSecret);

	var oauth_callback = oauth_callback;

	var windowRef = null;
	var loginCallback = null;

	var user = null;
	var _persister = null;
	var _loader = null;

	var credentials = {
		consumerKey: consumerKey
		,consumerSecret: consumerSecret
	};

	//only the first option (location) is used for Android, the rest are for the benefit of iOS
	var loginPopupOptions = [
		'location=yes'
		,'closebuttoncaption=Cancel'
		,'toolbar=yes'
		,'presentationstyle=pagesheet'
		,'transitionstyle=fliphorizontal'
	].join(',');

	function params(url){
		var parameters = [], parameter;
		var query = url.split('?');
		if (query.length > 1){
			query = query[1].split('&');
			for (var i = 0; i < query.length; i++) {
				parameter = query[i].split("=");
				if (parameter.length === 1) { parameter[1] = ""; }
				parameters[decodeURIComponent(parameter[0])] = decodeURIComponent(parameter[1]);
			}
		}
		return parameters;
	}

	function saveLogin(oauth_token, oauth_token_secret, screen_name, user_id){
		user = {
			oauth_token: oauth_token
			,oauth_token_secret: oauth_token_secret
			,screen_name: screen_name
			,user_id: user_id
		};
		//if a persistence method is set, call it
		if(_persister !== null) _persister.call(null, user);
		//set the oauth token and secret into codebird
		cb.setToken(oauth_token, oauth_token_secret);
	}

	function onLoadEvent(event){
		var url = event.url;
		var parameters = params(url);

		if (typeof(parameters.oauth_verifier) !== 'undefined'){
			var verifier = parameters.oauth_verifier;
			windowRef.close();
			cb.__call(
				"oauth_accessToken"
				,{ oauth_verifier: verifier }
				,function(reply){
					saveLogin(reply.oauth_token, reply.oauth_token_secret, reply.screen_name, reply.user_id);
					loginCallback.call(null, user);
				}
			);
		}else if(typeof(parameters.denied) !== 'undefined'){
			//login refused
			windowRef.close();
			loginCallback.call(null, null);
		}else{
			//probably an interstitial step during the login process; ignore
		}
	}

	var public_interface = {
		login: function(callback){
			loginCallback = callback;
			// cb.setConsumerKey(this.credentials.consumerKey, this.credentials.consumerSecret);
			cb.__call(
				"oauth_requestToken"
				,{ oauth_callback: oauth_callback }
				,function(reply){
					if (!reply.oauth_token){
						return callback(null);
					}
					cb.setToken(reply.oauth_token, reply.oauth_token_secret);
					cb.__call(
						"oauth_authorize"
						,{}
						,function(auth_url){
							windowRef = window.open(auth_url, '_blank', loginPopupOptions);
							windowRef.addEventListener('loadstart', onLoadEvent);
							windowRef.addEventListener('loadstop', onLoadEvent);
						}
					);
				}
			);
		}
		,loadConfigVia: function(fn){
			_loader = fn;
			var config = _loader.call(null);
			if (config){
				user = config;
				if (user !== null){
					cb.setToken(user.oauth_token, user.oauth_token_secret);
				}
			}
			return public_interface;
		}
		,persistConfigVia: function(fn){
			_persister = fn;
			if (user){
				_persister.call(null, user);
			}
			return public_interface;
		}
		,tweet: function(msg, callback){
			cb.__call(
				"statuses_update"
				,{"status": msg}
				,function (reply){
					callback(reply);
				}
			);
		}
	};

	return public_interface;
};
